#include "TArtCalibNEBULA.hh" 
#include "TArtNEBULAPla.hh"
#include "TArtNEBULAPlaPara.hh"
#include "TArtNEBULAParameters.hh"
#include "TArtRawEventObject.h"
#include "TArtCore.h" 
#include "TArtStoreManager.h"
#include "TArtMapper.h"

#include <TClonesArray.h>
#include <iostream>
#include <cmath>

//__________________________________________________________
TArtCalibNEBULA::TArtCalibNEBULA() : fReconstructed(false), fNEBULAParameters(0)
{
  TArtCore::Info(__FILE__,"Creating the NEBULA detector objects...");

  fStoreManager = TArtStoreManager::Instance();
  fNEBULAPlaArray = new TArtNEBULAPlaArray("TArtNEBULAPla", 300);
  fNEBULAPlaArray->SetOwner();
  fNEBULAPlaArray->SetName("NEBULAPla");
  fNEBULAPlaParaArray = new TArtNEBULAPlaParaArray("TArtNEBULAPlaPara", 300);
  fNEBULAPlaParaArray->SetOwner();
  fNEBULAPlaParaArray->SetName("NEBULAPlaPara");
  fStoreManager->AddDataContainer(fNEBULAPlaArray);
  fNEBULAPlaParaArray = new TArtNEBULAPlaParaArray;
}

//__________________________________________________________
TArtCalibNEBULA::~TArtCalibNEBULA()
{
  delete fNEBULAPlaParaArray;
  delete fNEBULAPlaArray;
}

//__________________________________________________________
void TArtCalibNEBULA::BeginEvent()
{
  if(!fNEBULAParameters){
    fNEBULAParameters = (const TArtNEBULAParameters*)fStoreManager->FindParameters("NEBULAParameters");
  }

  LoadRawData();
}

//__________________________________________________________
void TArtCalibNEBULA::LoadRawData()
{
  TArtRawEventObject* fEvent = (TArtRawEventObject*)fStoreManager->FindDataContainer("RawEvent");
  ClearData();

  for(int i=0; i<fEvent->GetNumSeg(); ++i){
    TArtRawSegmentObject* seg = fEvent->GetSegment(i);
    int device = seg->GetDevice();
    if(SAMURAI == device){
      for(int j=0; j<seg->GetNumData(); ++j){
	int fpl = seg->GetFP();
	int detector = seg->GetDetector();
	TArtRawDataObject* d = seg->GetData(j);
	int geo = d->GetGeo(); 
	int ch = d->GetCh(); 
	unsigned int val = d->GetVal();
	TArtRIDFMap mm(fpl,detector,geo,ch);
	const TArtNEBULAPlaPara* para = fNEBULAParameters->FindNEBULAPlaPara(mm);
	if(!para){
	  //	  TArtCore::Info(__FILE__,"Could not find TArtNEBULAPara...: Dev:%d, %s", device, mm.GetMapInfo());
	  TArtCore::Debug(__FILE__,"Could not find TArtNEBULAPara...: Dev:%d, %s", device, mm.GetMapInfo());
	  continue;
	}
	if(para->GetIgnore()){
	  TArtCore::Debug(__FILE__,"Ignore TArtNEBULAPara...: Dev:%d, %s", device, mm.GetMapInfo());
	  continue;
	}

	if(FindNEBULAPlaPara(para->GetID(), para->GetUDLR())){
	  int npara = fNEBULAPlaParaArray->GetEntries();
	  new ((*fNEBULAPlaParaArray)[npara]) TArtNEBULAPlaPara(*para);
	}
	TArtNEBULAPla* pla = FindNEBULAPla(para->GetID());
	if(!pla){
	  int npla = fNEBULAPlaArray->GetEntries();
	  pla = new ((*fNEBULAPlaArray)[npla]) TArtNEBULAPla();
	  pla->SetID(para->GetID());
	  pla->SetFPl(para->GetFPl());
	  pla->SetDetectorName(para->GetDetectorName());
	  pla->SetLayer(para->GetLayer());
	  pla->SetSubLayer(para->GetSubLayer());
	}

	TString udlr = para->GetUDLR();
	// if(detector == NEBULAT)
	if(detector == 41){
	  if(udlr == "U"){
	    pla->SetRawTU(val);
	  }else if(udlr == "D"){
	    pla->SetRawTD(val);
	  }
	}
	// if(detector == NEBULAQ)
	if(detector == 42){
	  if(udlr == "U"){
	    pla->SetRawQU(val);
	  }else if(udlr == "D"){
	    pla->SetRawQD(val);
	  }
	}
      }
    }
  }

  return;
}

//__________________________________________________________
void TArtCalibNEBULA::ReconstructData()
{
  for(int i=0;i<GetNumNEBULAPla();++i){
    TArtNEBULAPla* pla = GetNEBULAPla(i);
    int id = pla->GetID();
    const TArtNEBULAPlaPara* parau = fNEBULAParameters->FindNEBULAPlaPara(id, "U");
    if(!parau){
      TArtCore::Info(__FILE__,"cannot find para %d U", id);
      continue;
    }
    const TArtNEBULAPlaPara* parad = fNEBULAParameters->FindNEBULAPlaPara(id, "D");
    if(!parad){
      TArtCore::Info(__FILE__,"cannot find para %d D", id);
      continue;
    }

    int rawtu = pla->GetRawTU();
    int rawtd = pla->GetRawTD();
    int rawqu = pla->GetRawQU();
    int rawqd = pla->GetRawQD();

    int ihitmin = 0;
    if(rawtu>0 && rawtu<=4096) ++ihitmin;
    if(rawtd>0 && rawtd<=4096) ++ihitmin;      
    if(rawqu>0 && rawqu<=4096) ++ihitmin;
    if(rawqd>0 && rawqd<=4096) ++ihitmin;
    pla->SetIHitMin(ihitmin);

    if(ihitmin > 0){
      pla->SetDataState(1);
    }else{
      pla->SetDataState(-1);
      continue;
    }

    Double_t tu = rawtu * parau->GetTCal() + parau->GetTOffset();
    Double_t td = rawtd * parad->GetTCal() + parad->GetTOffset();
    Double_t eu = (rawqu - parau->GetQPed()) * parau->GetQCal();
    Double_t ed = (rawqd - parad->GetQPed()) * parad->GetQCal();

    Double_t t = (tu + td)/2;
    Double_t e = sqrt(eu*ed);

    pla->SetTime(t);
    pla->SetEnergy(e);
  }
  
  fReconstructed = true;
  return;
}

//__________________________________________________________
void TArtCalibNEBULA::EndEvent()
{
  ClearData();
}

//__________________________________________________________
Int_t TArtCalibNEBULA::GetNumNEBULAPla() const
{
  return fNEBULAPlaArray->GetEntries();
}

//__________________________________________________________
TArtNEBULAPla* TArtCalibNEBULA::GetNEBULAPla(int i) const
{
  return (TArtNEBULAPla*)fNEBULAPlaArray->At(i);
}

//__________________________________________________________
TArtNEBULAPla* TArtCalibNEBULA::FindNEBULAPla(Int_t id) const
{
  for(int i=0; i<GetNumNEBULAPla(); ++i){
    TArtNEBULAPla* pla = GetNEBULAPla(i);
    if(id == pla->GetID()) return pla;
  }
  return 0;
}

//__________________________________________________________
TArtNEBULAPlaArray* TArtCalibNEBULA::GetNEBULAPlaArray() const
{
  return fNEBULAPlaArray;
}

//__________________________________________________________
Int_t TArtCalibNEBULA::GetNumNEBULAPlaPara() const
{
  return fNEBULAPlaParaArray->GetEntries();
}

//__________________________________________________________
TArtNEBULAPlaPara* TArtCalibNEBULA::GetNEBULAPlaPara(int i) const
{
  return (TArtNEBULAPlaPara*)fNEBULAPlaParaArray->At(i);
  //  return (const TArtNEBULAPlaPara*)(*fNEBULAPlaParaArray)[i];
}

//__________________________________________________________
TArtNEBULAPlaPara* TArtCalibNEBULA::FindNEBULAPlaPara(int id, TString udlr) const
{
  for(int i=0; i<GetNumNEBULAPlaPara(); ++i){
    TArtNEBULAPlaPara* para = GetNEBULAPlaPara(i);
    if(id == para->GetID() && udlr == para->GetUDLR()) return para;
  }

  return 0;
  //  for(int i=0; i<GetNumNEBULAPlaPara(); ++i){
  //    const TArtNEBULAPlaPara* para = GetNEBULAPlaPara(i);
  //    std::cout << i << " " << para->GetID() << " " << para->GetUDLR() << std::endl;
  //    std::cout << i << " " << id << " " << udlr << std::endl;
  //    if(id == para->GetID() && udlr == para->GetUDLR()){
  //      return para;
  //    }
  //  }
  //  return 0;
}

//__________________________________________________________
TArtNEBULAPlaParaArray* TArtCalibNEBULA::GetNEBULAPlaParaArray() const
{
  return fNEBULAPlaParaArray;
}

//__________________________________________________________
void TArtCalibNEBULA::ClearData()
{
  fNEBULAPlaArray->Clear();
  fNEBULAPlaParaArray->Clear();
  fReconstructed = false;
  return;
}

